@@ -47,6 +47,36 @@ class FacebookProvider(Provider):
             'username': me.get('username')
         }
 
+    def autologin_script(self, service):
+        return """
+      $.getScript('//connect.facebook.net/%(lang)s/all.js', function(){
+        FB.init({
+          appId: '%(app_id)s',
+        });
+        FB.getLoginStatus(function (response) {
+            if (response.status == 'connected') {
+                $.ajax('/externalauth-userinfos', {
+                    data: {
+                        __externalauthprovider: "facebook",
+                        __externalauth_oauth2_token:
+                        response.authResponse.accessToken,
+                        autologin: true
+                    },
+                    dataType: 'json',
+                    success: function (data) {
+                        if (!data.anonymous) {
+                            document.location.reload();
+                        }
+                    }
+                });
+            }
+        });
+      });
+        """ % {
+            'lang': 'en_UK',
+            'app_id': service.application_id
+        }
+
 
 class ServiceAdapter(EntityAdapter):
     __regid__ = 'externalauth.service'
@@ -68,3 +98,6 @@ class ServiceAdapter(EntityAdapter):
             access_token_url=provider.access_token_url,
             base_url=provider.base_url,
         )
+
+    def autologin_script(self):
+        return self.provider.autologin_script(self.entity)
