@@ -22,7 +22,7 @@ from yams.buildobjs import (
     String, EntityType, SubjectRelation, Datetime, Boolean
 )
 
-from cubicweb.schema import ERQLExpression
+from cubicweb.schema import ERQLExpression, RRQLExpression
 
 
 # TODO set the maxsize of the String attributes.
@@ -87,7 +87,7 @@ class ExternalIdentity(EntityType):
     __permissions__ = {
         'read':   ('managers', ERQLExpression('X identity_of U')),
         'add':    ('managers',),
-        'update': ('managers',),
+        'update': ('managers', ERQLExpression('X identity_of U')),
         'delete': ('managers', ERQLExpression('X identity_of U'))
     }
     __unique_together__ = [('provider', 'uid')]
@@ -116,14 +116,23 @@ class ExternalIdentity(EntityType):
             'update': ('managers',),
         }
     )
+    autologin = Boolean(
+        required=True, default=False,
+        __permissions__={
+            'read': ('managers', 'users'),
+            'update': ('managers', ERQLExpression('X identity_of U')),
+        }
+    )
 
 
 class OAuth2Session(EntityType):
     __permissions__ = {
-        'read':   ('managers', ERQLExpression('X external_identity I, I identity_of U')),
+        'read':   ('managers',
+                   ERQLExpression('X external_identity I, I identity_of U')),
         'add':    ('managers',),
         'update': ('managers',),
-        'delete': ('managers', ERQLExpression('X external_identity I, I identity_of U'))
+        'delete': ('managers',
+                   ERQLExpression('X external_identity I, I identity_of U'))
     }
 
     external_identity = SubjectRelation(
