@@ -15,6 +15,7 @@
 
 """cubicweb-oauth views/forms/actions/components for web ui"""
 
+import json
 import re
 import urllib
 import urlparse
@@ -28,10 +29,12 @@ from cubicweb.view import View
 from cubicweb.web.controller import Controller
 from cubicweb.web.views import authentication
 from cubicweb.web.views import basecomponents, basetemplates
-from cubicweb.web import Redirect, formfields, formwidgets
+from cubicweb.web import Redirect, formfields, formwidgets, LogOut
 
 from cubicweb.web import InvalidSession
 
+from cubicweb import NoResultError, MultipleResultsError
+
 from cubes.oauth.authplugin import EXT_TOKEN
 
 _ = unicode
@@ -53,6 +56,34 @@ def get_next_login(login):
     return '{}.{}'.format(login, index)
 
 
+class ExternalAuthMixin(object):
+    def get_service(self, req, cnx, providername):
+        try:
+            return cnx.execute(
+                'ExternalAuthService S WHERE S provider P, P spid %(spid)s',
+                {'spid': providername}
+            ).one()
+        except (NoResultError, MultipleResultsError):
+            self.exception('no service for provider: %s' % providername)
+            raise Redirect(req.build_url(
+                __message='no service for provider %s' % providername
+            ))
+
+
+class ExternalAuthUserInfos(Controller):
+    __regid__ = 'externalauth-userinfos'
+
+    def publish(self, rset=None):
+        if self._cw.session.anonymous_session:
+            ret = {'anonymous': True}
+        else:
+            ret = {
+                'anonymous': False,
+                'login': self._cw.user.login
+            }
+        return json.dumps(ret)
+
+
 class ExternalAuthConfirm(Controller):
     __regid__ = 'externalauth-confirm'
     __select__ = match_form_params('code', '__externalauth_negociationid')
@@ -87,12 +118,8 @@ class ExternalAuthError(Controller):
             % (form.get('error_code'), form.get('error_message', ''))))
 
 
-class yes_marker(object):
-    def __bool__(self):
-        return True
-
-
-class ExternalAuthRetrieverStart(authentication.WebAuthInfoRetriever):
+class ExternalAuthRetrieverStart(
+        ExternalAuthMixin, authentication.WebAuthInfoRetriever):
     """External authentication first step.
 
     Will only redirect on the demanded service.
@@ -120,18 +147,7 @@ class ExternalAuthRetrieverStart(authentication.WebAuthInfoRetriever):
         vreg = self._cw  # YES, see cw/web/views/authentication.py ~106
         repo = vreg.config.repository(vreg)
         with repo.internal_cnx() as cnx:
-            servicerset = cnx.execute(
-                'ExternalAuthService S WHERE S provider P, P spid %(spid)s',
-                {'spid': providername}
-            )
-            if not servicerset:
-                self.error('no service for provider: %s' % providername)
-                raise Redirect(req.build_url(
-                    __message='no service for provider %s' % providername
-                ))
-
-            service = servicerset.one()
-
+            service = self.get_service(req, cnx, providername)
             path = req.relative_path(includeparams=True)
             # Remove the 'vid' parameter if the selected view is 'loggedout'
             if '?' in path:
@@ -160,7 +176,141 @@ class ExternalAuthRetrieverStart(authentication.WebAuthInfoRetriever):
             raise Redirect(url)
 
 
-class ExternalAuthRetrieverFinish(authentication.WebAuthInfoRetriever):
+class ExternalAuthRetrieverMixin(ExternalAuthMixin):
+    def _authentication_information(
+            self, req, cnx, adapter, service, oauth2_session, autologin=False):
+        try:
+            infos = adapter.provider.retrieve_info(oauth2_session)
+            extid_rset = cnx.execute(
+                'ExternalIdentity SI WHERE SI provider P, '
+                'P eid %(peid)s, SI uid %(uid)s',
+                {'peid': service.provider[0].eid, 'uid': infos['uid']}
+            )
+            assert extid_rset.rowcount < 2
+            if extid_rset:
+                external_identity = extid_rset.get_entity(0, 0)
+                if autologin and not external_identity.autologin:
+                    self.info(
+                        "Autologin is not enabled for identity %s",
+                        infos['uid'])
+                    raise authentication.NoAuthInfo
+            else:
+                external_identity = cnx.create_entity(
+                    'ExternalIdentity', provider=service.provider[0],
+                    uid=infos['uid']
+                )
+
+            # deactivate any previous access_token
+            cnx.execute(
+                'SET X active FALSE WHERE '
+                'X is OAuth2Session, '
+                'S external_identity SI, SI eid %(sieid)s, '
+                'S service SE, SE eid %(seid)s',
+                {'sieid': external_identity.eid, 'seid': service.eid}
+            )
+            oauth2_session = cnx.create_entity(
+                'OAuth2Session', service=service,
+                external_identity=external_identity,
+                access_token=oauth2_session.access_token,
+                active=True
+            )
+
+            cnx.commit(free_cnxset=False)
+        except authentication.NoAuthInfo:
+            raise
+        except Exception, e:
+            self.exception("Cannot get the oauth2_session")
+            raise authentication.NoAuthInfo(str(e))
+
+        if external_identity.identity_of:
+            user = external_identity.identity_of[0]
+            login = user.login
+            external_identity.cw_set(autologin=True)
+            cnx.commit()
+            self.info('identified %s on %s as %s' % (
+                user.login, service.provider[0].name, infos['uid']))
+        else:
+            self.debug(
+                "Unknown identity %s on %s: will create a local account",
+                infos['uid'], service.provider[0].name)
+
+            login = self.forge_login(cnx, infos)
+            password = make_uid()
+
+            # TODO insert more informations from the external service
+            # XXX We should loop around the call_service and retry with a
+            # different login (w. get_next_login) when it raises a
+            # ValidationError _or_ if the commit() raise a duplicate key
+            # error.
+
+            try:
+                user = cnx.call_service(
+                    'register_user',
+                    login=login,
+                    password=password,
+                    email=infos.get('email', None),
+                    firstname=infos.get('firstname', u''),
+                    surname=infos.get('lastname', u''))
+
+                cnx.execute(
+                    'SET X identity_of U '
+                    'WHERE X eid %(sieid)s, '
+                    'U is CWUser, U login %(login)s',
+                    {'sieid': external_identity.eid,
+                     'login': login}
+                )
+                external_identity.cw_set(autologin=True)
+                cnx.commit()
+                self.info('created user %s on %s as %s' % (
+                    user.login, service.provider[0].name, infos['uid']))
+            except:
+                self.exception("Cannot create user %s" % login)
+                raise
+
+        return login, {'__externalauth_directauth': EXT_TOKEN}
+
+
+class ExternalAuthRetrieverDirect(
+        ExternalAuthRetrieverMixin, authentication.WebAuthInfoRetriever):
+    __regid__ = 'externalauth-direct'
+
+    order = ExternalAuthRetrieverStart.order - 1
+
+    def request_has_auth_info(self, req):
+        return (
+            '__externalauthprovider' in req.form
+            and '__externalauth_oauth2_token' in req.form)
+
+    def revalidate_login(self, req):
+        # Always invalidate the current session
+        raise InvalidSession()
+
+    def authentication_information(self, req):
+        if not self.request_has_auth_info(req):
+            raise authentication.NoAuthInfo
+
+        spid = req.form.get('__externalauthprovider')
+        oauth2_token = req.form.get('__externalauth_oauth2_token')
+        autologin = bool(req.form.get('autologin'))
+
+        vreg = self._cw  # YES, see cw/web/views/authentication.py ~106
+        repo = vreg.config.repository(vreg)
+        with repo.internal_cnx() as cnx:
+            try:
+                service = self.get_service(req, cnx, spid)
+                adapter = service.cw_adapt_to('externalauth.service')
+                oauth2_session = adapter.oauth2_service.get_session(
+                    oauth2_token)
+            except:
+                self.error("Cannot get a auth2session")
+                raise authentication.NoAuthInfo
+            return self._authentication_information(
+                req, cnx, adapter, service, oauth2_session,
+                autologin=autologin)
+
+
+class ExternalAuthRetrieverFinish(
+        ExternalAuthRetrieverMixin, authentication.WebAuthInfoRetriever):
     __regid__ = 'externalauth-success'
     order = ExternalAuthRetrieverStart.order - 1
 
@@ -201,83 +351,11 @@ class ExternalAuthRetrieverFinish(authentication.WebAuthInfoRetriever):
                     'code': code,
                     'redirect_uri': nego['redirect_uri']
                 })
-
-                infos = adapter.provider.retrieve_info(oauth2_session)
-                extid_rset = cnx.execute(
-                    'ExternalIdentity SI WHERE SI provider P, '
-                    'P eid %(peid)s, SI uid %(uid)s',
-                    {'peid': service.provider[0].eid, 'uid': infos['uid']}
-                )
-                assert extid_rset.rowcount < 2
-                if extid_rset:
-                    external_identity = extid_rset.get_entity(0, 0)
-                else:
-                    external_identity = cnx.create_entity(
-                        'ExternalIdentity', provider=service.provider[0],
-                        uid=infos['uid']
-                    )
-
-                # deactivate any previous access_token
-                cnx.execute(
-                    'SET X active FALSE WHERE '
-                    'X is OAuth2Session, '
-                    'S external_identity SI, SI eid %(sieid)s, '
-                    'S service SE, SE eid %(seid)s',
-                    {'sieid': external_identity.eid, 'seid': service.eid}
-                )
-                oauth2_session = cnx.create_entity(
-                    'OAuth2Session', service=service,
-                    external_identity=external_identity,
-                    access_token=oauth2_session.access_token,
-                    active=True
-                )
-
-                cnx.commit(free_cnxset=False)
-            except Exception, e:
-                self.exception("Cannot get the oauth2_session")
-                raise authentication.NoAuthInfo(str(e))
-
-            if external_identity.identity_of:
-                user = external_identity.identity_of[0]
-                self.debug('identified %s on %s as %s' % (
-                    user.login, service.provider[0].name, infos['uid']))
-                login = user.login
-            else:
-                self.debug(
-                    "Unknown identity %s on %s: will create a local account",
-                    infos['uid'], service.provider[0].name)
-
-                login = self.forge_login(cnx, infos)
-                password = make_uid()
-
-                # TODO insert more informations from the external service
-                # XXX We should loop around the call_service and retry with a
-                # different login (w. get_next_login) when it raises a
-                # ValidationError _or_ if the commit() raise a duplicate key
-                # error.
-
-                try:
-                    cnx.call_service(
-                        'register_user',
-                        login=login,
-                        password=password,
-                        email=infos.get('email', None),
-                        firstname=infos.get('firstname', u''),
-                        surname=infos.get('lastname', u''))
-
-                    cnx.execute(
-                        'SET X identity_of U '
-                        'WHERE X eid %(sieid)s, '
-                        'U is CWUser, U login %(login)s',
-                        {'sieid': external_identity.eid,
-                         'login': login}
-                    )
-                    cnx.commit()
-                except:
-                    self.exception("Cannot create user %s" % login)
-                    raise
-
-        return login, {'__externalauth_directauth': EXT_TOKEN}
+            except:
+                self.error("Cannot get a auth2session")
+                raise authentication.NoAuthInfo
+            return self._authentication_information(
+                req, cnx, adapter, service, oauth2_session)
 
     def make_unique_login(self, cnx, login):
         """Return a login that does not yet exists in the database,
@@ -336,7 +414,7 @@ class ExternalAuthLogForm(basetemplates.BaseLogForm):
     onclick_args = ('externalauthlogbox', '__externalauthprovider')
 
 
-class ExternalAuthLoginLink(basecomponents.HeaderComponent):
+class ExternalAuthLoginLink(ExternalAuthMixin, basecomponents.HeaderComponent):
     __regid__ = 'externalauthlink'
     __select__ = (
         basecomponents.HeaderComponent.__select__ and
@@ -347,6 +425,15 @@ class ExternalAuthLoginLink(basecomponents.HeaderComponent):
     context = _('header-right')
     onclick = "javascript: cw.htmlhelpers.popupLoginBox('%s', '%s');"
 
+    def autologin_script(self):
+        with self._cw.cnx._repo.internal_cnx() as cnx:
+            for spid in ('facebook',):
+                self._cw.html_headers.add_post_inline_script(
+                    self.get_service(self._cw, cnx, spid).cw_adapt_to(
+                        'externalauth.service'
+                    ).autologin_script()
+                )
+
     def render(self, w):
         text = self._cw._('external auth login')
         w(u"""<a href="#" onclick="%s">%s</a>""" % (
@@ -357,6 +444,7 @@ class ExternalAuthLoginLink(basecomponents.HeaderComponent):
             'externalauthlogform', rset=self.cw_rset,
             id='externalauthlogbox', w=w
         )
+        self.autologin_script()
 
 
 class ExternalAuthLogFormView(View):
@@ -377,3 +465,17 @@ Select the provider you want to use to authenticate.
         form = self._cw.vreg['forms'].select('externalauthlogform', self._cw)
         form.render(w=w)
         w(u'</div>')
+
+
+# XXX Hijack the logout method.
+# We should have a hook for pre/post-logout actions
+def CookieSessionHandler_logout(self, req, goto_url):
+    if req.user.reverse_identity_of:
+        req.user.reverse_identity_of[0].cw_set(autologin=False)
+        req.cnx.commit()
+    self.session_manager.close_session(req.session)
+    req.remove_cookie(self.session_cookie(req))
+    raise LogOut(url=goto_url)
+
+from cubicweb.web.application import CookieSessionHandler
+CookieSessionHandler.logout = CookieSessionHandler_logout
